from .utils import register_model_alias_map

provider = "anthropic"
models = {
    "claude-3-haiku": "anthropic/claude-3-haiku-20240307",
    "claude-3.5-haiku": "anthropic/claude-3-5-haiku-20241022",
    "claude-4-sonnet": "anthropic/claude-sonnet-4-20250514",
    "claude-4-opus": "anthropic/claude-opus-4-20250514",
    "claude-4.1-opus": "anthropic/claude-opus-4-1-20250805",
    "claude-4.5-sonnet": "anthropic/claude-sonnet-4-5-20250929",
    "claude-4.5-haiku": "anthropic/claude-haiku-4-5-20251001",
    "claude-4.5-opus": "anthropic/claude-opus-4-5-20251101",
}

register_model_alias_map("anthropic", models)



from . import anthropic
from . import bedrock
from . import deepseek
from . import groq
from . import mistral
from . import openai
from . import replicate
from . import togetherai
from . import vertexai
from . import xai



from typing import Optional

from unify import BASE_URL
from unify.utils import http
from unify.utils.helpers import _create_request_header


def get_user_basic_info(*, api_key: Optional[str] = None):
    """
    Get basic information for the authenticated user.

    Args:
        api_key: If specified, unify API key to be used. Defaults
        to the value in the `UNIFY_KEY` environment variable.

    Returns:
        The basic information for the authenticated user.
    """
    headers = _create_request_header(api_key)
    response = http.get(f"{BASE_URL}/user/basic-info", headers=headers)
    return response.json()



from .utils import register_model_alias_map

models = {
    "gemini-3-pro": "vertex_ai/gemini-3-pro-preview",
    "gemini-2.5-flash-lite": "vertex_ai/gemini-2.5-flash-lite",
    "gemini-2.5-flash": "vertex_ai/gemini-2.5-flash",
    "gemini-2.5-pro": "vertex_ai/gemini-2.5-pro",
    "gemini-2.0-flash-lite": "vertex_ai/gemini-2.0-flash-lite-001",
    "gemini-2.0-flash": "vertex_ai/gemini-2.0-flash-001",
    "claude-3-haiku": "vertex_ai/claude-3-haiku@20240307",
    "claude-3.5-haiku": "vertex_ai/claude-3-5-haiku@20241022",
    "claude-4-sonnet": "vertex_ai/claude-sonnet-4@20250514",
    "claude-4-opus": "vertex_ai/claude-opus-4@20250514",
    "claude-4.1-opus": "vertex_ai/claude-opus-4-1@20250805",
    "claude-4.5-sonnet": "vertex_ai/claude-sonnet-4-5@20250929",
    "claude-4.5-haiku": "vertex_ai/claude-haiku-4-5@20251001",
    "claude-4.5-opus": "vertex_ai/claude-opus-4-5@20251101",
    "llama-3.1-405b-chat": "vertex_ai/meta/llama3-405b-instruct-maas",
    "llama-3.3-70b-chat": "vertex_ai/meta/llama-3.3-70b-instruct-maas",
    "llama-4-maverick-instruct": "vertex_ai/meta/llama-4-maverick-17b-128e-instruct-maas",
    "llama-4-scout-instruct": "vertex_ai/meta/llama-4-scout-17b-16e-instruct-maas",
    "mistral-medium": "vertex_ai/mistral-medium-3",
    "mistral-small": "vertex_ai/mistral-small-2503",
    "qwen-3-235b-a22b-instruct": "vertex_ai/qwen/qwen3-235b-a22b-instruct-2507-maas",
    "deepseek-v3.1": "vertex_ai/deepseek-ai/deepseek-v3.1-maas",
    "deepseek-r1": "vertex_ai/deepseek-ai/deepseek-r1-0528-maas",
    "gpt-oss-20b": "vertex_ai/openai/gpt-oss-20b-maas",
    "gpt-oss-120b": "vertex_ai/openai/gpt-oss-120b-maas",
}

register_model_alias_map("vertex-ai", models)



from typing import Any, Dict

_MODEL_ALIAS_MAP = {}


def register_model_alias_map(
    provider: str,
    model_map: Dict[str, Any],
) -> Dict[str, str]:
    _MODEL_ALIAS_MAP.update(
        {f"{model}@{provider}": alias for model, alias in model_map.items()},
    )


def get_model_alias(endpoint: str) -> str:
    """
    Get the alias for a model. If the model is not found, return the original model.

    Args:
        endpoint: The endpoint of the model.
    Returns:
        LiteLLM model name for the model.
    """
    return _MODEL_ALIAS_MAP.get(endpoint, endpoint)



from .utils import register_model_alias_map

models = {
    "gpt-oss-20b": "together_ai/openai/gpt-oss-20b",
    "gpt-oss-120b": "together_ai/openai/gpt-oss-120b",
    "deepseek-v3.1": "together_ai/deepseek-ai/DeepSeek-V3.1",
    "deepseek-r1": "together_ai/deepseek-ai/DeepSeek-R1",
    "deepseek-v3": "together_ai/deepseek-ai/DeepSeek-V3",
    "llama-4-maverick-instruct": "together_ai/meta-llama/Llama-4-Maverick-17B-128E-Instruct-FP8",
    "llama-3.3-70b-chat": "together_ai/meta-llama/Llama-3.3-70B-Instruct-Turbo",
    "llama-3.2-3b-chat": "together_ai/meta-llama/Llama-3.2-3B-Instruct-Turbo",
    "llama-3.1-70b-chat": "together_ai/meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo",
    "llama-3.1-405b-chat": "together_ai/meta-llama/Meta-Llama-3.1-405B-Instruct-Turbo",
    "mistral-small": "mistralai/Mistral-Small-24B-Instruct-2501",
    "qwen-3-235b-a22b-instruct": "together_ai/Qwen/Qwen3-235B-A22B-fp8-tput",
    "qwen-2.5-7b-instruct": "together_ai/Qwen/Qwen2.5-7B-Instruct-Turbo",
    "qwen-2.5-72b-instruct": "together_ai/Qwen/Qwen2.5-72B-Instruct-Turbo",
}
register_model_alias_map("togetherai", models)



from .utils import register_model_alias_map

models = {
    "llama-3.1-8b-chat": "groq/llama-3.1-8b-instant",
    "llama-3.3-70b-chat": "groq/llama-3.3-70b-versatile",
    "llama-4-maverick-instruct": "groq/meta-llama/llama-4-maverick-17b-128e-instruct",
    "llama-4-scout-instruct": "groq/meta-llama/llama-4-scout-17b-16e-instruct",
    "gpt-oss-20b": "groq/openai/gpt-oss-20b",
    "gpt-oss-120b": "groq/openai/gpt-oss-120b",
}

register_model_alias_map("groq", models)



from .utils import register_model_alias_map

models = {
    "llama-4-maverick-instruct": "replicate/meta/llama-4-maverick-instruct",
    "llama-4-scout-instruct": "replicate/meta/llama-4-scout-instruct",
    "llama-3-8b-chat": "replicate/meta/meta-llama-3-8b-instruct",
    "llama-3-70b-chat": "replicate/meta/meta-llama-3-70b-instruct",
    "llama-3.1-405b-chat": "replicate/meta/meta-llama-3.1-405b-instruct",
    "deepseek-v3.1": "replicate/deepseek-ai/deepseek-v3.1",
    "deepseek-v3": "replicate/deepseek-ai/deepseek-v3",
    "deepseek-r1": "replicate/deepseek-ai/deepseek-r1",
    "o4-mini": "replicate/openai/o4-mini",
    "gpt-4.1": "replicate/openai/gpt-4.1",
    "gpt-4.1-mini": "replicate/openai/gpt-4.1-mini",
    "gpt-4.1-nano": "replicate/openai/gpt-4.1-nano",
    "gpt-5": "replicate/openai/gpt-5",
    "gpt-5-mini": "replicate/openai/gpt-5-mini",
    "gpt-5-nano": "replicate/openai/gpt-5-nano",
    "gpt-oss-20b": "replicate/openai/gpt-oss-20b",
    "gpt-oss-120b": "replicate/openai/gpt-oss-120b",
    "gpt-5.1": "replicate/openai/gpt-5.1",
    "claude-4.5-haiku": "replicate/anthropic/claude-4.5-haiku",
    "claude-4.5-sonnet": "replicate/anthropic/claude-4.5-sonnet",
    "claude-4-sonnet": "replicate/anthropic/claude-4-sonnet",
    "claude-3.5-haiku": "replicate/anthropic/claude-3.5-haiku",
    "gemini-3-pro": "replicate/google/gemini-3-pro",
}

register_model_alias_map("replicate", models)



"""Provider-specific message preprocessing applied before cache lookup."""

import copy
import json
from typing import Any, Dict, List, Optional, Tuple

CONCURRENT_USER_MESSAGES_EXPLANATION = (
    "For all user messages which are represented in JSON format, please treat each "
    "item in the list as a separate message. The user message is shown in JSON format "
    "because this API does not natively support concurrent user messages (which is "
    "what actually occurred), and concurrent user messages are being represented this "
    "way instead."
)


def _move_system_messages_to_front(
    messages: List[Dict[str, Any]],
) -> List[Dict[str, Any]]:
    """Move all system messages to the beginning, preserving relative order."""
    system = [m for m in messages if m.get("role") == "system"]
    non_system = [m for m in messages if m.get("role") != "system"]
    return system + non_system


def _combine_adjacent_user_messages(
    messages: List[Dict[str, Any]],
) -> Tuple[List[Dict[str, Any]], bool]:
    """
    Combine adjacent user messages into JSON content array format.
    Returns (result, whether_any_combining_occurred).
    """
    if not messages:
        return [], False

    result = []
    combined_any = False
    i = 0

    while i < len(messages):
        msg = messages[i]

        if msg.get("role") != "user":
            result.append(msg)
            i += 1
            continue

        # Collect adjacent user messages
        user_contents = []
        while i < len(messages) and messages[i].get("role") == "user":
            user_contents.append(messages[i].get("content", ""))
            i += 1

        if len(user_contents) == 1:
            result.append(msg)
        else:
            combined_any = True
            result.append(
                {
                    "role": "user",
                    "content": json.dumps(
                        {
                            "role": "user",
                            "content": [
                                {"type": "text", "text": c} for c in user_contents
                            ],
                        },
                        indent=4,
                    ),
                },
            )

    return result, combined_any


def apply_provider_preprocessing(
    kw: Dict[str, Any],
    provider: Optional[str],
) -> Dict[str, Any]:
    """Apply provider-specific preprocessing to messages in kw dict (mutates kw)."""
    messages = kw.get("messages")
    if not messages:
        return kw

    # Only Anthropic preprocessing for now
    first_provider = (provider or "").split("->")[0].strip()
    if first_provider != "anthropic":
        return kw

    messages = copy.deepcopy(messages)
    messages = _move_system_messages_to_front(messages)
    messages, combined_any = _combine_adjacent_user_messages(messages)

    if combined_any:
        # Insert explanation after system messages
        insert_pos = sum(1 for m in messages if m.get("role") == "system")
        messages.insert(
            insert_pos,
            {
                "role": "system",
                "content": CONCURRENT_USER_MESSAGES_EXPLANATION,
            },
        )

    kw["messages"] = messages
    return kw



from .utils import register_model_alias_map

models = {
    "gpt-oss-20b": "bedrock/us.openai.gpt-oss-20b-1:0",
    "gpt-oss-120b": "bedrock/us.openai.gpt-oss-120b-1:0",
    "deepseek-r1": "bedrock/us.deepseek.r1-v1:0",
    "llama-3.3-70b-chat": "bedrock/us.meta.llama3-3-70b-instruct-v1:0",
    "llama-3.2-1b-chat": "bedrock/us.meta.llama3-2-1b-instruct-v1:0",
    "llama-3.2-3b-chat": "bedrock/us.meta.llama3-2-3b-instruct-v1:0",
    "llama-3.1-8b-chat": "bedrock/meta.llama3-1-8b-instruct-v1:0",
    "llama-3.1-70b-chat": "bedrock/meta.llama3-1-70b-instruct-v1:0",
    "llama-3.1-405b-chat": "bedrock/meta.llama3-1-405b-instruct-v1:0",
    "llama-3-8b-chat": "bedrock/meta.llama3-8b-instruct-v1:0",
    "llama-3-70b-chat": "bedrock/meta.llama3-70b-instruct-v1:0",
    "claude-3-haiku": "bedrock/us.anthropic.claude-3-haiku-20240307-v1:0",
    "claude-3.5-haiku": "bedrock/us.anthropic.claude-3-5-haiku-20241022-v1:0",
    "claude-4-sonnet": "bedrock/us.anthropic.claude-sonnet-4-20250514-v1:0",
    "claude-4-opus": "bedrock/us.anthropic.claude-opus-4-20250514-v1:0",
    "claude-4.1-opus": "bedrock/us.anthropic.claude-opus-4-1-20250805-v1:0",
    "claude-4.5-sonnet": "bedrock/us.anthropic.claude-sonnet-4-5-20250929-v1:0",
    "claude-4.5-opus": "bedrock/us.anthropic.claude-opus-4-5-20251101-v1:0",
}

register_model_alias_map("bedrock", models)



from .utils import register_model_alias_map

models = {
    "gpt-3.5-turbo": "gpt-3.5-turbo",
    "gpt-4": "gpt-4",
    "gpt-4-turbo": "gpt-4-turbo",
    "gpt-4o": "gpt-4o",
    "gpt-4o-2024-05-13": "gpt-4o-2024-05-13",
    "gpt-4o-mini": "gpt-4o-mini",
    "chatgpt-4o-latest": "chatgpt-4o-latest",
    "o1": "o1",
    "o3-mini": "o3-mini",
    "gpt-4o-search-preview": "gpt-4o-search-preview",
    "gpt-4o-mini-search-preview": "gpt-4o-mini-search-preview",
    "gpt-4.1": "gpt-4.1",
    "gpt-4.1-mini": "gpt-4.1-mini",
    "gpt-4.1-nano": "gpt-4.1-nano",
    "o3": "o3",
    "o4-mini": "o4-mini",
    "gpt-5": "gpt-5",
    "gpt-5-mini": "gpt-5-mini",
    "gpt-5-nano": "gpt-5-nano",
    "gpt-5-chat-latest": "gpt-5-chat-latest",
    "gpt-5.1": "gpt-5.1",
    "gpt-5.1-chat-latest": "gpt-5.1-chat-latest",
}

register_model_alias_map("openai", models)



from .utils import register_model_alias_map

models = {
    "grok-4.1-fast-reasoning": "xai/grok-4.1-fast-reasoning",
    "grok-4.1-fast-non-reasoning": "xai/grok-4.1-fast-non-reasoning",
    "grok-code-fast": "xai/grok-code-fast-1",
    "grok-4-fast-reasoning": "xai/grok-4-fast-reasoning",
    "grok-4-fast-non-reasoning": "xai/grok-4-fast-non-reasoning",
    "grok-4": "xai/grok-4",
    "grok-3": "xai/grok-3",
    "grok-3-mini": "xai/grok-3-mini",
}

register_model_alias_map("xai", models)



from .utils import register_model_alias_map

models = {
    "mistral-large": "mistral/mistral-large-latest",
    "mistral-medium": "mistral/mistral-medium-latest",
    "mistral-small": "mistral/mistral-small-latest",
}

register_model_alias_map("mistral", models)



import json
import logging
import os
from functools import wraps
from typing import Callable

import requests
from requests.adapters import HTTPAdapter
from urllib3 import Retry

_LOGGER = logging.getLogger("unify_requests")
_LOG_ENABLED = os.getenv("UNIFY_REQUESTS_DEBUG", "false").lower() in ("true", "1")
_LOGGER.setLevel(logging.DEBUG if _LOG_ENABLED else logging.WARNING)

_SESSION = requests.Session()
_RETRIES = Retry(total=5, backoff_factor=0.1, status_forcelist=[500, 502, 503, 504])
_ADAPTER = HTTPAdapter(max_retries=_RETRIES, pool_connections=20, pool_maxsize=20)
_SESSION.mount("http://", _ADAPTER)
_SESSION.mount("https://", _ADAPTER)


class RequestError(Exception):
    def __init__(self, url: str, r_type: str, response: requests.Response, /, **kwargs):
        super().__init__(
            f"{r_type}:{url} with {kwargs} failed with status code {response.status_code}: {response.text}",
        )
        self.response = response


def _log(type: str, url: str, mask_key: bool = True, /, **kwargs):
    _kwargs_str = ""
    if mask_key and "headers" in kwargs:
        key = kwargs["headers"]["Authorization"]
        kwargs["headers"]["Authorization"] = "***"

    for k, v in kwargs.items():
        if isinstance(v, dict):
            _kwargs_str += f"{k:}:{json.dumps(v, indent=2)},\n"
        else:
            _kwargs_str += f"{k}:{v},\n"

    if mask_key and "headers" in kwargs:
        kwargs["headers"]["Authorization"] = key

    log_msg = f"""
====== {type} =======
url:{url}
{_kwargs_str}
"""
    _LOGGER.debug(log_msg)


def _mask_auth_key(kwargs: dict):
    if "headers" in kwargs:
        kwargs["headers"]["Authorization"] = "***"
    return kwargs


def _log_request_if_enabled(fn: Callable) -> Callable:
    """
    Only wrap request function if logging is enabled.
    """
    if not _LOG_ENABLED:
        return fn

    @wraps(fn)
    def inner(method, url, **kwargs):
        _log(f"{method}", url, True, **kwargs)
        res: requests.Response = fn(method, url, **kwargs)
        try:
            _log(f"{method} response:{res.status_code}", url, response=res.json())
        except requests.exceptions.JSONDecodeError:
            _log(f"{method} response:{res.status_code}", url, response=res.text)
        return res

    return inner


@_log_request_if_enabled
def request(method, url, raise_for_status=True, **kwargs) -> requests.Response:
    try:
        res = _SESSION.request(method, url, **kwargs)
        if raise_for_status:
            res.raise_for_status()
        return res
    except requests.exceptions.HTTPError as e:
        kwargs = _mask_auth_key(kwargs)
        raise RequestError(url, method, e.response, **kwargs)


def get(url, params=None, **kwargs):
    return request("GET", url, params=params, **kwargs)


def options(url, **kwargs):
    return request("OPTIONS", url, **kwargs)


def head(url, **kwargs):
    return request("HEAD", url, **kwargs)


def post(url, data=None, json=None, **kwargs):
    return request("POST", url, data=data, json=json, **kwargs)


def put(url, data=None, **kwargs):
    return request("PUT", url, data=data, **kwargs)


def patch(url, data=None, **kwargs):
    return request("PATCH", url, data=data, **kwargs)


def delete(url, **kwargs):
    return request("DELETE", url, **kwargs)
